<?php
/**
 * @Copyright
 *
 * @package js_weather class *
 * @author     Joachim Schmidt {@link http://www.jschmidt-systemberatung.de/}
 * @version Version: 1.0.0 - March 2023
 * @desciption Application to provide weather forcast weather data ftom openWeatherMap
 * @link       Project Site {@link http://www.jschmidt-systemberatung.de/}
 *
 * @license GNU/GPL
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * change activity:
 */

defined ( '_JEXEC' ) or die ( 'Restricted access' );

include_once (dirname(__FILE__) . '/settings.php');
 
use Joomla\CMS\Factory;
use jsweather\js_weather;
include_once (dirname(__DIR__) . '/js_weather.php');

$input = Factory::getApplication()->input;

if  ($input->exists('lang'))
    $lang = $input->get('lang', 'en', 'STRING') ;

$text = parse_ini_file(dirname(__DIR__) . "/weather.ini", true);
$valid_languages = $text['languages'];
if (strpos($valid_languages, $lang) === false)
    $lang = "en";
$text = $text[$lang];

if  ($input->exists('location'))
    $location = $input->get('location', '', 'STRING') ;
 
if  ($input->exists('use_ip'))
    $use_ip = $input->get('use_ip', '0', 'BOOLEAN') ;

$cachetime = $cachetime2;
if  ($input->exists('cache'))
    $cachetime = $input->get('cache', '', 'INT') ;

$dir = JPATH_BASE . "/cache/js_weather";
if ($cachetime > 0)
{
    if (! file_exists($dir))
        mkdir(JPATH_BASE . '/cache/js_weather', 0755);
}
$file = $dir . "/save_weatherdata." . $lang . "1";

if (file_exists($file) && $cachetime > 0)
    $changedTime = filemtime($file);
else
    $changedTime = time();

$currentTime = time();

$myweather = new js_weather($key, false);
$myweather->setLanguage($lang);
$myweather->setID($lang . "1");

$rc = $myweather->setLocation($location, $use_ip);
if ($rc == false)
{
    printf ( $text['location_error'], $location);
    return;
}

if ($myweather->getUseIP() == true)
   $cachetime = 0;

if ($cachetime > 0)
{
    if (! file_exists($file) || ($currentTime - $changedTime) > $cachetime)
    {
        $weather_info = json_decode($myweather->get_weather_data($dir));
          
        if ($weather_info->cod != 200)
        {
           echo $text['error'] . errorMsg($weather_info->cod, $text, $weather_info->message);
           return;
        }
        else
            $changedTime = time();
    }
    else
    {
        $fp = fopen($file, "r");
        $data = fread($fp, filesize($file));
        $weather_info = json_decode($data);
        $changedTime = filemtime($file);
    }
}
else
{
    $weather_info = json_decode($myweather->get_weather_data());
    $changedTime = time();
        
    if ($weather_info->cod != 200)
    {
        echo $text['error'] .  errorMsg($weather_info->cod, $text, $weather_info->message);
        return;
    }
}

$date = setmyDate($lang, $text, $changedTime);
$wind_dir = getWindDirection($text, $weather_info->wind->deg);
$weather_info->main->pressure = $myweather->format_number($weather_info->main->pressure);
$weather_info->visibility = $myweather->format_number($weather_info->visibility);
$weather_info->wind->speed = $myweather->format_number($weather_info->wind->speed, 1);

?>
<link href="<?php echo $app_root; ?>/jsweather/css/weather.css" rel="stylesheet" />
<div>
<div style="width:90%; background-image: url(<?php echo $app_root; ?>/jsweather/images/clouds.png);" class="current_weather_container">
 <h3 style="font-size: 1.15em; text-align: center;"><?php echo $text['weather'] . " in " . $weather_info->name . " (" . $weather_info->sys->country . ")"; ?></h3>
  <div class="my_weather-time">
   <div><?php echo $date . " - " . date("G:i ", $changedTime); ?></div>
   <div class="my_weather-forecast"><?php echo ucwords($weather_info->weather[0]->description); ?></div>
</div>
  
<table class="my_current_weather-table">
 <tr>
 
<td style="vertical-align: middle; padding-right: 15px;">
  <div class="my_weather-forecast">
   <table class="my_current_weather-table">
   <tr>
   <td style="vertical-align: top;"><img src="https://openweathermap.org/img/w/<?php echo $weather_info->weather[0]->icon; ?>.png"></td>
   <td style="vertical-align: middle;"> <?php  echo round($weather_info->main->temp,0);?>°C
   </td>
   <td style="vertical-align: middle;"><span class="my_min-temperature"><?php echo round($weather_info->main->temp_min,0); ?>°C min</span></td>
   <td style="vertical-align: middle;"><span class="my_min-temperature"><?php echo round($weather_info->main->temp_max,0); ?>°C max</span></td>
   </tr>
   </table>
  </div>
 </td>
 
  <td style="padding-right: 25px;vertical-align: middle;">
  <div>
   <table class="my_current_weather-table">
    <tr>
    <td style="vertical-align: bottom;"><img src="<?php echo $app_root; ?>/jsweather/images/sunrise.png" title="<?php echo $text['sunrise']; ?>" style="width:35px;"></td>
    <td style="vertical-align: middle; padding-right: 10px; "><?php echo " " . date("G:i ", $weather_info->sys->sunrise); ?></td>
    <td style="vertical-align: bottom;"><img src="<?php echo $app_root; ?>/jsweather/images/sunset.png" title="<?php echo $text['sunset']; ?>" style="width:30px;"></td>
    <td style="vertical-align: middle;"><?php echo " " . date("G:i ", $weather_info->sys->sunset); ?></td>
    </tr>
   </table>
  </div>
 </td>
    
 <td style="padding-left: 10px; padding-right: 10px;">
 <div class="my_weather-misc" style="color: #070f5e;">
  
   <div><?php echo $text['humidity']; ?> <?php echo $weather_info->main->humidity; ?> %</div>
   <div>Wind: <?php echo $weather_info->wind->speed; ?> km/h <?php echo $wind_dir; ?></div>
   <div><?php echo $text['pressure']; ?> <?php echo $weather_info->main->pressure; ?> hPa</div>
   <div><?php echo $text['visibility']; ?> <?php echo $weather_info->visibility; ?>m</div>
  </div>
  </td>
  

</tr>
</table>

</div>

<?php
function errorMsg ($code, $text, $msg)
{
    switch ($code)
    {
        case 400:
            return "(400) " . $text['400'] . ", <b>Response:</b> " . $msg;
            break;
        case 401:
            return "(401) " . $text['401'];
            break;
        case 404:
            return "(404) " . $text['404'];
            break;
        case 429:
            return "(429) " . $text['429'];
            break;
        default:
            if ($code == null)
              return "(nav) " . $text['nav'];
            else
              return "(" . $code . ") " . $msg;
    }
}

function getWindDirection($text, $deg)
{
    $winddir = [ $text["wind_N"],
                 $text["wind_NE"], $text["wind_E"], $text["wind_SE"], $text["wind_S"],
                 $text["wind_SW"], $text["wind_W"], $text["wind_NW"], $text["wind_N"] ];
    
    return $winddir[round($deg*8/360)];
}

function setmyDate($lang, $text,  $time)
{
    if ($lang =='en')
       $date =  date("jS F, Y", $time);
    else
       $date = date("j. ",$time) . $text['month'][date("n", $time)] . date( " Y", $time);

    return $date;
}

?>

