<?php
/**
 * @filesource: line_temperature.php
 * @author:     Joachim Schmidt - joachim.schmidt@jschmidt-systemberatung.de
 * @copyright:  Copyright (C) 2016 Joachim Schmidt. All rights reserved.
 * @license:    GNU/GPL
 * @version:    1.0
 * @abstract:   script to use php-helper classes for php_RGraph package
 *
 */
include_once "environment.php";
use phpRGraph\rgraph_chart;
use phpRGraph\rgraph_options;

$lang = "de";
if (array_key_exists('lang', $_GET))
    $lang = $_GET['lang'];
 else
    $ang = 'en';

$text = parse_ini_file(dirname(__DIR__) . "/weather.ini", true);
$valid_languages = $text['languages'];
if (strpos($valid_languages, $lang) === false)
    $lang = "en";
$text = $text[$lang];

$template = "default.php";
$draw_option = "draw();";
$width = "475";
$height = "220";

if (array_key_exists('data', $_GET))
$url_data = json_decode($_GET['data'], true);
else
{
    echo $text['chart_error1'];
    return;
}

if (json_last_error() != null)
{
    echo $text['chart_error2'];
    return;
}

$dir =  dirname(__FILE__);

$chart = new rgraph_chart("cvs", null, "Line");
$chart->set_data($url_data['data']);
$options = new rgraph_options($dir . "/line.ini");
$options->set_option("title", $text['chart_title']);

$options->set_option("yaxisScaleMin", min(array(
        min($url_data['data'][0]),
        min($url_data['data'][1]) )) - 1);
$options->set_option("yaxisScaleMax", max(array(
        max($url_data['data'][0]),
        max($url_data['data'][1]) )) + 1);

if ($options->get_option("yaxisScaleMax") == 0)
    $options->set_option("yaxisScaleMax", $options->get_option("yaxisScaleMax") + 2);
$options->set_option("key", array( "9:00 " . $text['hour'], "15:00 " . $text['hour'] ));
$options->set_option("keyInteractive", true);
$options->set_option("keyPositionX", 380);
$options->set_option("keyPositionY", 8);
$labels = $url_data['labels'];
$options->set_option("xaxisLabels", $labels);
$options->set_option("yaxisScaleUnitsPost", "°C");
$options->set_option("filledRange", true);
$options->set_option("filled", true);
$options->set_option("filledColors", array( $options->setOpacity("#00BFFF", 0.3) ));

$options->set_option("xaxisLabels", $labels);
$options->set_tooltip_formatted($chart->get_data(), "<b>#key#</b><br>#label#: #val# °C", "Line");

$chart->set_options($options);

$rgraph_json = $chart->toString();

$chart_script = "";
$event_script = "";

include_once ($template);

?>



